@@ -2,28 +2,29 @@
 
 namespace Icinga\Module\Businessprocess\Controllers;
 
+use Icinga\Date\DateFormatter;
 use Icinga\Module\Businessprocess\BpConfig;
-use Icinga\Module\Businessprocess\State\MonitoringState;
-use Icinga\Module\Businessprocess\Storage\ConfigDiff;
-use Icinga\Module\Businessprocess\Html\Element;
-use Icinga\Module\Businessprocess\Html\HtmlString;
-use Icinga\Module\Businessprocess\Html\HtmlTag;
-use Icinga\Module\Businessprocess\Html\Icon;
-use Icinga\Module\Businessprocess\Html\Link;
+use Icinga\Module\Businessprocess\BpNode;
 use Icinga\Module\Businessprocess\Node;
 use Icinga\Module\Businessprocess\Renderer\Breadcrumb;
 use Icinga\Module\Businessprocess\Renderer\Renderer;
 use Icinga\Module\Businessprocess\Renderer\TileRenderer;
 use Icinga\Module\Businessprocess\Renderer\TreeRenderer;
 use Icinga\Module\Businessprocess\Simulation;
+use Icinga\Module\Businessprocess\State\MonitoringState;
+use Icinga\Module\Businessprocess\Storage\ConfigDiff;
 use Icinga\Module\Businessprocess\Storage\LegacyConfigRenderer;
 use Icinga\Module\Businessprocess\Web\Component\ActionBar;
 use Icinga\Module\Businessprocess\Web\Component\RenderedProcessActionBar;
 use Icinga\Module\Businessprocess\Web\Component\Tabs;
 use Icinga\Module\Businessprocess\Web\Controller;
-use Icinga\Module\Businessprocess\Web\Url;
+use Icinga\Util\Json;
 use Icinga\Web\Notification;
+use Icinga\Web\Url;
 use Icinga\Web\Widget\Tabextension\DashboardAction;
+use Icinga\Web\Widget\Tabextension\OutputFormat;
+use ipl\Html\Html;
+use ipl\Html\HtmlString;
 
 class ProcessController extends Controller
 {
@@ -41,7 +42,7 @@ class ProcessController extends Controller
         $this->setTitle($title);
         $this->controls()
             ->add($this->tabsForCreate()->activate('create'))
-            ->add(HtmlTag::h1($title));
+            ->add(Html::tag('h1', null, $title));
 
         $this->content()->add(
             $this->loadForm('bpConfig')
@@ -62,7 +63,7 @@ class ProcessController extends Controller
         $this->setTitle($title);
         $this->controls()
             ->add($this->tabsForCreate()->activate('upload'))
-            ->add(HtmlTag::h1($title));
+            ->add(Html::tag('h1', null, $title));
 
         $this->content()->add(
             $this->loadForm('BpUpload')
@@ -87,22 +88,33 @@ class ProcessController extends Controller
 
         $renderer = $this->prepareRenderer($bp, $node);
 
-        if ($this->params->get('unlocked')) {
-            $renderer->unlock();
+        if (! $this->showFullscreen && ($node === null || ! $renderer->rendersImportedNode())) {
+            if ($this->params->get('unlocked')) {
+                $renderer->unlock();
+            }
+
+            if ($bp->isEmpty() && $renderer->isLocked()) {
+                $this->redirectNow($this->url()->with('unlocked', true));
+            }
         }
 
-        if ($bp->isEmpty() && $renderer->isLocked()) {
-            $this->redirectNow($this->url()->with('unlocked', true));
-        }
+        $this->handleFormatRequest($bp, $node);
 
         $this->prepareControls($bp, $renderer);
+
+        $this->tabs()->extend(new OutputFormat());
+
         $missing = $bp->getMissingChildren();
         if (! empty($missing)) {
-            $bp->addError(sprintf('There are missing nodes: %s', implode(', ', $missing)));
+            if (($count = count($missing)) > 10) {
+                $missing = array_slice($missing, 0, 10);
+                $missing[] = '...';
+            }
+            $bp->addError('There are %d missing nodes: %s', $count, implode(', ', $missing));
         }
-        $this->content()->addContent($this->showHints($bp));
-        $this->content()->addContent($this->showWarnings($bp));
-        $this->content()->addContent($this->showErrors($bp));
+        $this->content()->add($this->showHints($bp));
+        $this->content()->add($this->showWarnings($bp));
+        $this->content()->add($this->showErrors($bp));
         $this->content()->add($renderer);
         $this->loadActionForm($bp, $node);
         $this->setDynamicAutorefresh();
@@ -113,29 +125,24 @@ class ProcessController extends Controller
         $controls = $this->controls();
 
         if ($this->showFullscreen) {
-            $controls->attributes()->add('class', 'want-fullscreen');
-            $controls->add(
-                Link::create(
-                    Icon::create('resize-small'),
-                    $this->url()->without('showFullscreen')->without('view'),
-                    null,
-                    array(
-                        'style' => 'float: right',
-                        'title' => $this->translate(
-                            'Leave full screen and switch back to normal mode'
-                        )
-                    )
-                )
-            );
+            $controls->getAttributes()->add('class', 'want-fullscreen');
+            $controls->add(Html::tag(
+                'a',
+                [
+                    'href'  => $this->url()->without('showFullscreen')->without('view'),
+                    'title' => $this->translate('Leave full screen and switch back to normal mode'),
+                    'style' => 'float: right'
+                ],
+                Html::tag('i', ['class' => 'icon icon-resize-small'])
+            ));
         }
 
         if (! ($this->showFullscreen || $this->view->compact)) {
             $controls->add($this->getProcessTabs($bp, $renderer));
+            $controls->getAttributes()->add('class', 'separated');
         }
-        if (! $this->view->compact) {
-            $controls->add(Element::create('h1')->setContent($this->view->title));
-        }
-        $controls->add(Breadcrumb::create($renderer));
+
+        $controls->add(Breadcrumb::create(clone $renderer));
         if (! $this->showFullscreen && ! $this->view->compact) {
             $controls->add(
                 new RenderedProcessActionBar($bp, $renderer, $this->Auth(), $this->url())
@@ -209,32 +216,52 @@ class ProcessController extends Controller
 
         if ($action === 'add' && $canEdit) {
             $form = $this->loadForm('AddNode')
+                ->setSuccessUrl(Url::fromRequest()->without('action'))
+                ->setStorage($this->storage())
                 ->setProcess($bp)
                 ->setParentNode($node)
                 ->setSession($this->session())
                 ->handleRequest();
+        } elseif ($action === 'editmonitored' && $canEdit) {
+            $form = $this->loadForm('EditNode')
+                ->setSuccessUrl(Url::fromRequest()->without('action'))
+                ->setProcess($bp)
+                ->setNode($bp->getNode($this->params->get('editmonitorednode')))
+                ->setParentNode($node)
+                ->setSession($this->session())
+                ->handleRequest();
         } elseif ($action === 'delete' && $canEdit) {
-                $form =$this->loadForm('DeleteNode')
-                    ->setProcess($bp)
-                    ->setNode($bp->getNode($this->params->get('deletenode')))
-                    ->setParentNode($node)
-                    ->setSession($this->session())
-                    ->handleRequest();
+            $form = $this->loadForm('DeleteNode')
+                ->setSuccessUrl(Url::fromRequest()->without('action'))
+                ->setProcess($bp)
+                ->setNode($bp->getNode($this->params->get('deletenode')))
+                ->setParentNode($node)
+                ->setSession($this->session())
+                ->handleRequest();
         } elseif ($action === 'edit' && $canEdit) {
-            $form =$this->loadForm('Process')
+            $form = $this->loadForm('Process')
+                ->setSuccessUrl(Url::fromRequest()->without('action'))
                 ->setProcess($bp)
                 ->setNode($bp->getNode($this->params->get('editnode')))
                 ->setSession($this->session())
                 ->handleRequest();
         } elseif ($action === 'simulation') {
             $form = $this->loadForm('simulation')
+                ->setSuccessUrl(Url::fromRequest()->without('action'))
                 ->setNode($bp->getNode($this->params->get('simulationnode')))
                 ->setSimulation(Simulation::fromSession($this->session()))
                 ->handleRequest();
+        } elseif ($action === 'move') {
+            $form = $this->loadForm('MoveNode')
+                ->setProcess($bp)
+                ->setParentNode($node)
+                ->setSession($this->session())
+                ->setNode($bp->getNode($this->params->get('movenode')))
+                ->handleRequest();
         }
 
         if ($form) {
-            $this->content()->prependContent(HtmlString::create((string) $form));
+            $this->content()->prepend(HtmlString::create((string) $form));
         }
     }
 
@@ -258,9 +285,9 @@ class ProcessController extends Controller
     protected function showWarnings(BpConfig $bp)
     {
         if ($bp->hasWarnings()) {
-            $ul = Element::create('ul', array('class' => 'warning'));
+            $ul = Html::tag('ul', array('class' => 'warning'));
             foreach ($bp->getWarnings() as $warning) {
-                $ul->createElement('li')->addContent($warning);
+                $ul->add(Html::tag('li')->setContent($warning));
             }
 
             return $ul;
@@ -272,9 +299,9 @@ class ProcessController extends Controller
     protected function showErrors(BpConfig $bp)
     {
         if ($bp->hasWarnings()) {
-            $ul = Element::create('ul', array('class' => 'error'));
+            $ul = Html::tag('ul', array('class' => 'error'));
             foreach ($bp->getErrors() as $msg) {
-                $ul->createElement('li')->addContent($msg);
+                $ul->add(Html::tag('li')->setContent($msg));
             }
 
             return $ul;
@@ -285,40 +312,44 @@ class ProcessController extends Controller
 
     protected function showHints(BpConfig $bp)
     {
-        $ul = Element::create('ul', array('class' => 'error'));
+        $ul = Html::tag('ul', ['class' => 'error']);
         foreach ($bp->getErrors() as $error) {
-            $ul->createElement('li')->addContent($error);
+            $ul->add(Html::tag('li')->setContent($error));
         }
         if ($bp->hasChanges()) {
-            $ul->createElement('li')->setSeparator(' ')->addContent(sprintf(
+            $li = Html::tag('li')->setSeparator(' ');
+            $li->add(sprintf(
                 $this->translate('This process has %d pending change(s).'),
                 $bp->countChanges()
-            ))->addContent(
-                Link::create(
-                    $this->translate('Store'),
-                    'businessprocess/process/config',
-                    array('config' => $bp->getName())
-                )
-            )->addContent(
-                Link::create(
-                    $this->translate('Dismiss'),
-                    $this->url()->with('dismissChanges', true),
-                    null
-                )
-            );
+            ))->add(Html::tag(
+                'a',
+                [
+                    'href' => Url::fromPath('businessprocess/process/config')
+                        ->setParams($this->getRequest()->getUrl()->getParams())
+                ],
+                $this->translate('Store')
+            ))->add(Html::tag(
+                'a',
+                ['href' => $this->url()->with('dismissChanges', true)],
+                $this->translate('Dismiss')
+            ));
+            $ul->add($li);
         }
 
         if ($bp->hasSimulations()) {
-            $ul->createElement('li')->setSeparator(' ')->addContent(sprintf(
+            $li = Html::tag('li')->setSeparator(' ');
+            $li->add(sprintf(
                 $this->translate('This process shows %d simulated state(s).'),
                 $bp->countSimulations()
-            ))->addContent(Link::create(
-                $this->translate('Dismiss'),
-                $this->url()->with('dismissSimulations', true)
+            ))->add(Html::tag(
+                'a',
+                ['href' => $this->url()->with('dismissSimulations', true)],
+                $this->translate('Dismiss')
             ));
+            $ul->add($li);
         }
 
-        if ($ul->hasContent()) {
+        if (! $ul->isEmpty()) {
             return $ul;
         } else {
             return null;
@@ -355,7 +386,7 @@ class ProcessController extends Controller
         $this->setTitle($title);
         $this->controls()
             ->add($this->tabsForConfig($bp)->activate('source'))
-            ->add(HtmlTag::h1($title))
+            ->add(Html::tag('h1', null, $title))
             ->add($this->createConfigActionBar($bp, $showDiff));
 
         $this->setViewScript('process/source');
@@ -399,13 +430,11 @@ class ProcessController extends Controller
         $this->setTitle($title);
         $this->controls()
             ->add($this->tabsForConfig($bp)->activate('config'))
-            ->add(HtmlTag::h1($title))
+            ->add(Html::tag('h1', null, $title))
             ->add($this->createConfigActionBar($bp));
 
-        $url = Url::fromPath(
-            'businessprocess/process/show?unlocked',
-            array('config' => $bp->getName())
-        );
+        $url = Url::fromPath('businessprocess/process/show')
+            ->setParams($this->getRequest()->getUrl()->getParams());
         $this->content()->add(
             $this->loadForm('bpConfig')
                 ->setProcessConfig($bp)
@@ -421,48 +450,42 @@ class ProcessController extends Controller
 
         if ($showDiff) {
             $params = array('config' => $config->getName());
-            $actionBar->add(
-                Link::create(
-                    $this->translate('Source'),
-                    'businessprocess/process/source',
-                    $params,
-                    array(
-                        'class' => 'icon-doc-text',
-                        'title' => $this->translate('Show source code'),
-                    )
-                )
-            );
+            $actionBar->add(Html::tag(
+                'a',
+                [
+                    'href'  => Url::fromPath('businessprocess/process/source', $params),
+                    'class' => 'icon-doc-text',
+                    'title' => $this->translate('Show source code')
+                ],
+                $this->translate('Source')
+            ));
         } else {
             $params = array(
                 'config'   => $config->getName(),
                 'showDiff' => true
             );
 
-            $actionBar->add(
-                Link::create(
-                    $this->translate('Diff'),
-                    'businessprocess/process/source',
-                    $params,
-                    array(
-                        'class' => 'icon-flapping',
-                        'title' => $this->translate('Highlight changes'),
-                    )
-                )
-            );
+            $actionBar->add(Html::tag(
+                'a',
+                [
+                    'href'  => Url::fromPath('businessprocess/process/source', $params),
+                    'class' => 'icon-flapping',
+                    'title' => $this->translate('Highlight changes')
+                ],
+                $this->translate('Diff')
+            ));
         }
 
-        $actionBar->add(
-            Link::create(
-                $this->translate('Download'),
-                'businessprocess/process/download',
-                array('config' => $config->getName()),
-                array(
-                    'target' => '_blank',
-                    'class'  => 'icon-download',
-                    'title'  => $this->translate('Download process configuration')
-                )
-            )
-        );
+        $actionBar->add(Html::tag(
+            'a',
+            [
+                'href'      => Url::fromPath('businessprocess/process/download', ['config' => $config->getName()]),
+                'class'     => 'icon-download',
+                'target'    => '_blank',
+                'title'     => $this->translate('Download process configuration')
+            ],
+            $this->translate('Download')
+        ));
 
         return $actionBar;
     }
@@ -511,4 +534,65 @@ class ProcessController extends Controller
 
         return $tabs;
     }
+
+    protected function handleFormatRequest(BpConfig $bp, BpNode $node = null)
+    {
+        $desiredContentType = $this->getRequest()->getHeader('Accept');
+        if ($desiredContentType === 'application/json') {
+            $desiredFormat = 'json';
+        } elseif ($desiredContentType === 'text/csv') {
+            $desiredFormat = 'csv';
+        } else {
+            $desiredFormat = strtolower($this->params->get('format', 'html'));
+        }
+
+        switch ($desiredFormat) {
+            case 'json':
+                $response = $this->getResponse();
+                $response
+                    ->setHeader('Content-Type', 'application/json')
+                    ->setHeader('Cache-Control', 'no-store')
+                    ->setHeader(
+                        'Content-Disposition',
+                        'inline; filename=' . $this->getRequest()->getActionName() . '.json'
+                    )
+                    ->appendBody(Json::sanitize($node !== null ? $node->toArray() : $bp->toArray()))
+                    ->sendResponse();
+                exit;
+            case 'csv':
+                $csv = fopen('php://temp', 'w');
+
+                fputcsv($csv, ['Path', 'Name', 'State', 'Since']);
+
+                foreach ($node !== null ? $node->toArray(null, true) : $bp->toArray(true) as $node) {
+                    $data = [$node['path'], $node['name']];
+
+                    if (isset($node['state'])) {
+                        $data[] = $node['state'];
+                    }
+
+                    if (isset($node['since'])) {
+                        $data[] = DateFormatter::formatDateTime($node['since']);
+                    }
+
+                    fputcsv($csv, $data);
+                }
+
+                $response = $this->getResponse();
+                $response
+                    ->setHeader('Content-Type', 'text/csv')
+                    ->setHeader('Cache-Control', 'no-store')
+                    ->setHeader(
+                        'Content-Disposition',
+                        'attachment; filename=' . $this->getRequest()->getActionName() . '.csv'
+                    )
+                    ->sendHeaders();
+
+                rewind($csv);
+
+                fpassthru($csv);
+
+                exit;
+        }
+    }
 }
