@@ -2,8 +2,8 @@
 
 namespace Icinga\Module\Businessprocess;
 
-use Icinga\Module\Businessprocess\Html\Link;
 use Icinga\Module\Businessprocess\Web\Url;
+use ipl\Html\Html;
 
 class HostNode extends MonitoredNode
 {
@@ -32,11 +32,12 @@ class HostNode extends MonitoredNode
 
     protected $className = 'host';
 
-    public function __construct(BpConfig $bp, $object)
+    protected $icon = 'host';
+
+    public function __construct($object)
     {
         $this->name     = $object->hostname . ';Hoststatus';
         $this->hostname = $object->hostname;
-        $this->bp       = $bp;
         if (isset($object->state)) {
             $this->setState($object->state);
         } else {
@@ -44,11 +45,6 @@ class HostNode extends MonitoredNode
         }
     }
 
-    public function getAlias()
-    {
-        return $this->getHostname();
-    }
-
     public function getHostname()
     {
         return $this->hostname;
@@ -60,15 +56,10 @@ class HostNode extends MonitoredNode
             'host' => $this->getHostname(),
         );
 
-        if ($this->bp->hasBackendName()) {
-            $params['backend'] = $this->bp->getBackendName();
+        if ($this->getBpConfig()->hasBackendName()) {
+            $params['backend'] = $this->getBpConfig()->getBackendName();
         }
 
-        return Url::fromPath('monitoring/host/show', $params);
-    }
-
-    public function getLink()
-    {
-        return Link::create($this->hostname, $this->getUrl());
+        return Url::fromPath('businessprocess/host/show', $params);
     }
 }
