@@ -35,7 +35,7 @@ class ServiceNodeTest extends BaseTestCase
     public function testRendersCorrectLink()
     {
         $this->assertEquals(
-            '<a href="/icingaweb2/monitoring/service/show?host=localhost&amp;service=ping%20%3C%3E%20pong">'
+            '<a href="/icingaweb2/businessprocess/service/show?host=localhost&amp;service=ping%20%3C%3E%20pong">'
             . 'localhost: ping &lt;&gt; pong</a>',
             $this->pingOnLocalhost()->getLink()->render()
         );
@@ -47,10 +47,10 @@ class ServiceNodeTest extends BaseTestCase
     protected function pingOnLocalhost()
     {
         $bp = new BpConfig();
-        return new ServiceNode($bp, (object) array(
+        return (new ServiceNode((object) array(
             'hostname' => 'localhost',
             'service'  => 'ping <> pong',
             'state'    => 0,
-        ));
+        )))->setBpConfig($bp)->setHostAlias('localhost')->setAlias('ping <> pong');
     }
 }
